/* Fitxer: data1.c
   Descripci: Definici de les funcions per gestionar el tipus "data"
   Autor: Isidre Guix
*/

#include "data1.h"
#include "conio.h"
#include "verscomp.h"
#include <stdio.h>
#include <string.h>

int esAnyTraspas (int any)
{
	if (any<=0) return 0;
	return (any%4==0 && (any%100!=0 || any%400==0));
}

int dataOK (int dia, int mes, int any)
{
	if (dia<=0 || mes<=0 || mes>12 || any<=0) return 0;
	switch (mes)
	{
		case 1: case 3: case 5: case 7: case 8: case 10: case 12:
			if (dia>31) return 0;
			break;
		case 4: case 6: case 9: case 11:
			if (dia>30) return 0;
			break;
		case 2:
			if (dia>29) return 0;
			if (esAnyTraspas(any)==0 && dia>28) return 0;
	}
	return 1;
}

int inputData (struct t_data *pdata)
{
	char aux[11];
	unsigned int i;
	do
	{ i = scanf ("%10[^\n]",aux); neteja_stdin();}
	while (i == 0);
	/* En aquest moment, la data introduda per l'usuari s a la variable aux.
	   Com que tenim la funci cadenaToData() que sap omplir una data amb el 
	   contingut d'una cadena, controlant els possibles errors, no ens cal fer
	   altra cosa que invocar-la per aconseguir el nostre propsit */
	return cadenaToData(aux,pdata);
}

void outputData(struct t_data data)
{
	printf ("%02d-%02d-%04d", data.dia, data.mes, data.any);
}

int getDia (struct t_data data)
{	return data.dia; }

int getMes (struct t_data data)
{	return data.mes; }

int getAny (struct t_data data)
{	return data.any; }

void dataToCadena (char *cadena, struct t_data data)
{
	sprintf(cadena,"%02u-%02u-%04u",data.dia,data.mes,data.any);
}

int cadenaToData (char *cadena, struct t_data *pdata)
{
	int dia, mes, any;
	int i;
	if (strlen(cadena)!=10) return 0;
	if (!strchr(" .-/",cadena[2]) || !strchr(" .-/",cadena[5])) return 0;
	dia = 0;
	for (i=0;i<2;i++)
		if (cadena[i]<'0' || cadena[i]>'9') return 0;
		else dia = dia * 10 + cadena[i] - 48;
	mes = 0;
	for (i=3;i<5;i++)
		if (cadena[i]<'0' || cadena[i]>'9') return 0;
		else mes = mes * 10 + cadena[i] - 48;
	any = 0;
	for (i=6;i<10;i++)
		if (cadena[i]<'0' || cadena[i]>'9') return 0;
		else any = any * 10 + cadena[i] - 48;
	/* En aquests moments ja tenim els valors numrics corresponents a la data*/
	if (dataOK(dia,mes,any))
	{	pdata->dia = dia; pdata->mes = mes; pdata->any = any;
		return 1;
	} else return 0;
}

int compararDates (struct t_data data1, struct t_data data2)
{	
	if (data1.any==data2.any && data1.mes==data2.mes && data1.dia==data2.dia)
		return 0;
	if (data1.any<data2.any ||
		(data1.any==data2.any && (data1.mes<data2.mes ||
	                              (data1.mes==data2.mes && data1.dia<data2.dia)
	                             )
	    )
	   ) return -1;
	return 1;
}

int diesEntreDates (struct t_data data1, struct t_data data2)
{
	int result;
	int factor;
	struct t_data dg,dp;	/* Per identificar la data gran i la data petita */

	if (compararDates(data1,data2)<0) { dg=data2; dp=data1; factor=-1; }
	else {dg=data1; dp=data2; factor=1;}

	/* dg conte la data gran i dp conte la data petita */
	for (result=0;compararDates(dp,dg);result++,dp=dataSumaDies(dp,1));
	return result*factor;
}

struct t_data dataSumaDies (struct t_data data, int numDies)
{
	struct t_data resultat;
	int dia=data.dia+numDies;
	int mes=data.mes;
	int any=data.any;
	while (dataOK(dia,mes,any)==0)
	{
		if (dia>0)
		{	switch (mes)
			{
				case 4: case 6: case 9: case 11:
					dia=dia-30; mes++;
					break;
				case 1: case 3: case 5: case 7: case 8: case 10: case 12:
					dia=dia-31; mes++;
					if (mes==13) {mes = 1; any++; }
					break;
				case 2: 	if (esAnyTraspas(any)) dia=dia-29;
							else dia=dia-28;
							mes++;
			}
		}
		else
		{	switch (mes)
			{
				case 1: case 2: case 4: case 6: case 8: case 9: case 11:
					dia=dia+31; mes--;
					if (mes==0) {mes = 12; any--; }
					break;
				case 5: case 7: case 10: case 12:
					dia=dia+30; mes--;
					break;
				case 3: 	if (esAnyTraspas(any)) dia=dia+29;
							else dia=dia+28;
							mes--;
			}
		}
	}
	resultat.dia=dia; resultat.mes=mes; resultat.any=any;
	return resultat;
}
