/* Fitxer: data1.h
   Descripci: Definici del tipus de dada "data" i declaraci de les funcions
               per gestionar-la.
			   S'afegeix alguna funci que t a veure amb les dates, com per 
			   exemple, saber si un any s o no de trasps
   Autor: Isidre Guix
*/

#ifndef __DATA__H__
#define __DATA__H__

struct t_data
{
	unsigned char dia, mes;
	unsigned short any;
};

int esAnyTraspas (int any);
/* Comprova si el valor de l'argument correspon o no a un any de trasps
   Retorna: 0: No s any de trasps
            1: S s any de trasps
*/

int dataOK (int dia, int mes, int any);
/* Comprova si la combinaci dels arguments "dia", "mes" i "any" corresponen
   a una data correcta.
   Retorna: 0: No s data correcta
            1: S s data correcta
*/

int inputData (struct t_data *pdata);
/*	Efectua, per l'entrada estndard, la lectura de la data en format dd-mm-aaaa
	on el separador pot ser '.', '-' o '/'.
	Significat dels arguments:
		pdata: Variable que recollir la data introduda per l'usuari
	Significat del valor retornat:
		0: Data incorrecta - La variable mant el valor inicial
		1: Data correcta
*/

void outputData (struct t_data data);
/* Visualitza, per la sortida estndard, la data en format dd-mm-aaaa */

int getDia (struct t_data data);
/* Retorna el dia de la data de l'argument */

int getMes (struct t_data data);
/* Retorna el mes de la data de l'argument */

int getAny (struct t_data data);
/* Retorna l'any de la data de l'argument */

void dataToCadena (char *cadena, struct t_data data);
/*	Efectua la conversi d'una data a cadena, en format dd-mm-aaaa
	Significat dels arguments:
		data  : Variable que cont la data a convertir
		cadena: Variable que recull la conversi de la data
                s responsabilitat del programador que tingui capacitat per a
                10 carcters (ms 1 per '\0')
*/

int cadenaToData (char *cadena, struct t_data *pdata);
/*	Omple una variable de tipus data a partir del contingut d'una cadena, que
    es suposa cont la data en format dd-mm-aaaa on el separador pot ser '.',
    '-' o '/'.
	Significat dels arguments:
		pdata -> Variable que recollir la data existent a la cadena
	Significat del valor retornat:
		0 -> Data incorrecta - La variable pdata mant el valor inicial
		1 -> Data correcta
*/

int compararDates (struct t_data data1, struct t_data data2);
/*	Efectua la comparaci de dues dates, amb els resultats:
		0: Sn iguals
		-1: data1 s menor que data 2
		+1: data1 s major que data 2
*/

int diesEntreDates (struct t_data data1, struct t_data data2);
/*  Efectua la diferncia de data1 menys data2, obtenint com a resultat el 
    nmero de dies que hi ha entre les dues:
		S'obt el resultat 0 si data1==data2
		S'obt un nmero positiu si data1 > data2
		S'obt un nmero negatiu si data1 < data2
*/

struct t_data dataSumaDies (struct t_data data, int numDies);
/*	Retorna una data corresponent al resultat de sumar els dies de l'argument
	"numDies" a la data de l'argument "data".
	L'argument "numDies" pot ser positiu, negatiu o zero, i per tant, el resultat
	ser una data posterior, igual o anterior a la data de l'argument "data"
*/
#endif
